import static org.junit.Assert.*;

import org.junit.Test;

public class TestStackAndQueue {

	@Test
	public void testArrayedStack() {
		ArrayedStack s = new ArrayedStack();
		assertTrue(s.size() == 0 && s.isEmpty());
		
		try {
			String top = s.top();
			fail("Empty stack should have caused an exception.");
		}
		catch(IllegalArgumentException e) { }
		
		s.push("Alan");
		s.push("Mark");
		s.push("Tom");
		assertTrue(s.size() == 3 && !s.isEmpty());
		assertEquals("Tom", s.top());
		
		String oldTop = s.pop();
		assertEquals("Tom", oldTop);
		String newTop = s.top();
		assertEquals("Mark", newTop);
		
		oldTop = s.pop();
		assertEquals("Mark", oldTop);
		newTop = s.top();
		assertEquals("Alan", newTop);
	}
	
	@Test
	public void testReverseViaStack() {
		String[] names = {"Alan", "Mark", "Tom"};
		String[] reverseOfNames = {"Tom", "Mark", "Alan"}; 
		StackUtilities.reverse(names);
		assertArrayEquals(reverseOfNames, names);
	}
	
	@Test
	public void testMatchingDelimiters() {
		assertTrue(StackUtilities.isMatched(""));
		assertTrue(StackUtilities.isMatched("{[()]}"));
		assertTrue(StackUtilities.isMatched("{[()]}({[]})"));
		assertFalse(StackUtilities.isMatched("{[]})"));
		assertFalse(StackUtilities.isMatched("{[])"));
		assertFalse(StackUtilities.isMatched("{[]"));
	}
	
	@Test
	public void testArrayedQueue() {
		ArrayedQueue q = new ArrayedQueue();
		assertTrue(q.size() == 0 && q.isEmpty());
		
		try {
			String first = q.first();
			fail("Empty queue should have caused an exception.");
		}
		catch(IllegalArgumentException e) { }
		
		q.enqueue("Alan");
		q.enqueue("Mark");
		q.enqueue("Tom");
		assertTrue(q.size() == 3 && !q.isEmpty());
		assertEquals("Alan", q.first());
		
		String oldFirst = q.dequeue();
		assertEquals("Alan", oldFirst);
		String newFirst = q.first();
		assertEquals("Mark", newFirst);
		
		oldFirst = q.dequeue();
		assertEquals("Mark", oldFirst);
		newFirst = q.first();
		assertEquals("Tom", newFirst);
	}
}
